#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>

/* sc2000.dat file extractor Copyright 1994 by John Stewart see the file
   sc2kutil.txt for info on this program.

   to contact me(the author) try the following:

email: jstewart@sdl.psych.wright.edu or
       jstewart@desire.wright.edu

smail: John Stewart
       3115 Seven Oaks Dr.
       Laura, OH  45337

phone: 513-947-1109

*/
  

#define MAX_FILES 500

#define MAX_FNAME_LEN 13

typedef struct
{
	unsigned long offset_arr[MAX_FILES];
	char fname_arr[MAX_FILES][MAX_FNAME_LEN];
	unsigned int num_files;
} pfhi_t; /* packed file header info type */


long filesize(FILE *stream)
{
   long curpos, length;

   curpos = ftell(stream);
   fseek(stream, 0L, SEEK_END);
   length = ftell(stream);
   fseek(stream, curpos, SEEK_SET);
   return length;
}


void read_header(char *fname, pfhi_t *header_info)
{
unsigned int file_num;
FILE *infp;
unsigned int i;
char tmp_fname[MAX_FNAME_LEN];

unsigned long int first_offset;
unsigned long int temp_offset;

	infp=fopen(fname,"rb");

	file_num=0;

	/* do set up i.e. read first entry to find end of header block */

	for(i=0;i<12;i++)
		tmp_fname[i]=fgetc(infp); /* read in 12 chars */

	fread(&first_offset, sizeof(unsigned long int), 1, infp);

	header_info->offset_arr[file_num]=first_offset;
	strcpy(header_info->fname_arr[file_num],tmp_fname);

	file_num=1;

	while( first_offset>ftell(infp) ) /* do this loop while we haven't */
							    /* seeked to the first_offset */
	{
		for(i=0;i<12;i++)
			tmp_fname[i]=fgetc(infp); /* read in 12 chars */

		tmp_fname[12]=0;

		fread(&temp_offset, sizeof(unsigned long int), 1, infp);

		header_info->offset_arr[file_num]=temp_offset;
		strcpy(header_info->fname_arr[file_num],tmp_fname);
		file_num++;

	}
	header_info->offset_arr[file_num]=filesize(infp);

	header_info->num_files=file_num;

	fclose(infp);
}

void view_header(char *fname, pfhi_t *header_info)
{
unsigned int i;

	printf("unpaking files from: %s\n",fname);

	for(i=0;i<header_info->num_files;i++)
	{
		printf("\tfname: %s, offset=%lu\n",
			  header_info->fname_arr[i],
			  header_info->offset_arr[i]);
	}
}

void unpak_file(char *fname, pfhi_t *header_info)
{
FILE *infp;
int file_num;

FILE *outfp;

unsigned char *buffer;

unsigned long int num_bytes;

	infp=fopen(fname,"rb");

	buffer=malloc(32768);

	for(file_num=0;file_num<header_info->num_files; file_num++)
	{
		if( ( outfp=fopen(header_info->fname_arr[file_num],"wb") )==NULL)
		{
			printf("Error opening file: %s\n",
				  header_info->fname_arr[file_num]);
		}
		else
			printf("extracting: %s at offset=%lu\n",
				  header_info->fname_arr[file_num],
				  header_info->offset_arr[file_num]);




		num_bytes=header_info->offset_arr[file_num+1]-
				header_info->offset_arr[file_num];

		fseek(infp,header_info->offset_arr[file_num],SEEK_SET);

		while(num_bytes)
		{
			fread(buffer, 32768, 1, infp);

			if(num_bytes > 32767)
			{
				fwrite(buffer, 32768, 1, outfp);
				num_bytes-=32768;
			}
			else
			{
				fwrite(buffer, num_bytes, 1, outfp);
				num_bytes=0;
			}
		}
		fclose(outfp);
	}

	fclose(infp);
	free(buffer);
}

void main(int argc, char *argv[])
{

pfhi_t header_info;

if(argc!=2)
{
	printf("Usage: %s fname.pak\n",argv[0]);
	printf("where fname.pak is the pak file you wish to unpak.\n");
	exit(-1);
}

read_header(argv[1],&header_info);
/*view_header(argv[1],&header_info);*/
unpak_file(argv[1],&header_info);

}